//**************************************************************************//
//                                                                          //
// Copyright (c) 1997.                                                      //
//      Richard D. Irwin, Inc.                                              //
//                                                                          //
// This software may not be distributed further without permission from     //
// Richard D. Irwin, Inc.                                                   //
//                                                                          //
// This software is distributed WITHOUT ANY WARRANTY. No claims are made    //
// as to its functionality or purpose.                                      //
//                                                                          //
// Authors: James P. Cohoon and Jack W. Davidson                            //
// Date: 7/15/96                                                            //
// Version: 1.0b                                                            //
//                                                                          //
//**************************************************************************//

#include <assert.h>
#include <stdlib.h>
#include <ctype.h>   // this library name is changing in the proposed standard to cctype
#include <string.h>  // this library name is changing in the proposed standard to cstring
#include "bignum.h"

// Insert(): facilitator for BigNum insertion
void BigNum::Insert(ostream &sout) const {
	for (int i = NumberDigits; i >= 1; --i) {
		sout << GetDigit(i);
	}
}

// auxiliary operator << for BigNums
ostream& operator<<(ostream &sout, const BigNum &n){
	n.Insert(sout);
	return sout;
}

// default BigNum constructor sets number to zero
BigNum::BigNum() {
	SetNumberDigits(1);
	GetMemory(1);
	SetDigit(1, 0);
}

// BigNum constructor from another BigNum
BigNum::BigNum(const BigNum &n) {
	SetNumberDigits(n.GetNumberDigits());
	GetMemory(GetNumberDigits());
	for (int i = 1; i <= GetNumberDigits(); ++i) {
		SetDigit(i, n.GetDigit(i));
	}
}

// BigNum constructor that takes string of digits
BigNum::BigNum(const char s[]) {
	if (strlen(s) == 0) { // empty string is 0
		SetNumberDigits(1);
 		GetMemory(1);
		SetDigit(1, 0);
	}
	else { // have nonempty string to process
		// process digits
		int nd = strlen(s);
		SetNumberDigits(nd);
		GetMemory(nd);
		for (int i = 0; i < nd; ++i) {
			if (isdigit(s[i])) { // digit
				SetDigit(nd - i, s[i] - '0');
			}
	 		else { // non-digit
				cerr << "Bad number: " << s << endl;
				exit(1);
			}
		}
		RemoveLeadingZeros();
	}
}

// GetNumberDigits(): set the size of the number
int BigNum::GetNumberDigits() const {
	return NumberDigits;
}

// SetNumberDigits(): set the size of the number
void BigNum::SetNumberDigits(int n) {
	assert (n > 0);
	NumberDigits = n;
}

// GetDigit(): return the desired digit value
int BigNum::GetDigit(int i) const {
	assert ((i > 0) && (i <= GetNumberDigits()));
	return Digits[i-1];
}

// SetDigit(): set desired digit to desired value
void BigNum::SetDigit(int i, int n) {
	assert((i > 0) && (i <= GetNumberDigits()));
	assert((n >= 0) && (n <= 9));
	Digits[i-1] = n;
}

// GetMemory(): get space for digits
void BigNum::GetMemory(int n) {
	assert(n > 0);
	Digits = new int [n];
	if (Digits == 0) {
		cerr << "Out of memory" << endl;
		exit(1);
	}
}

// Bignum destructor
BigNum::~BigNum() {
	delete [] Digits;
}

// ResizeMemory(): reset the space to hold the new
// desired number of digits
void BigNum::ResizeMemory(int n) {
	assert(n > 0);
	delete [] Digits;
	GetMemory(n);
}

// operator =: assignment for BigNum
BigNum& BigNum::operator=(const BigNum &n) {
	if (this != &n) {
		ResizeMemory(n.GetNumberDigits());
		SetNumberDigits(n.GetNumberDigits());
		for (int i = 1; i <= GetNumberDigits(); ++i) {
	 		SetDigit(i, n.GetDigit(i));
		}
	}
	return *this;
}

// Add(): facilitator for BigNum addition
BigNum BigNum::Add(const BigNum &n) const {
	BigNum Result;
	int nd; // maximum number of digits in sum
	if (GetNumberDigits() < n.GetNumberDigits())
		nd = n.GetNumberDigits() + 1;
	else
		nd = GetNumberDigits() + 1;
	Result.ResizeMemory(nd);
	Result.SetNumberDigits(nd);
	int Carry = 0;
	for (int i = 1; i <= nd; ++i) {
		int d1; // current digit of left operand
		int d2; // current digit of right operand
		if (i <= GetNumberDigits())
			d1 = GetDigit(i);
		else
			d1 = 0;
		if (i <= n.GetNumberDigits())
			d2 = n.GetDigit(i);
		else
			d2 = 0;
		Result.SetDigit(i, (d1 + d2 + Carry) % 10);
		Carry = (d1 + d2 + Carry) / 10;
	}
	Result.RemoveLeadingZeros();
	return Result;
}

// Multiply (): facilitator for BigNum multiplication
BigNum BigNum::Multiply(const BigNum &n) const {
	BigNum Product;
	for (int i = 1; i <= GetNumberDigits(); ++i) {
		BigNum Term = n.Scale(GetDigit(i));
		BigNum ShiftedTerm = Term.LeftDigitShift(i-1);
		Product = Product + ShiftedTerm;
	}
	return Product;
}


// RemoveLeadingZeros(): remove zeros in front of most
// most significant digit
void BigNum::RemoveLeadingZeros() {
	int nd = GetNumberDigits();
	while ((nd > 1) && (GetDigit(nd) == 0)) {
		--nd;
	}
	if (nd != GetNumberDigits()) {
		int *OldDigits = Digits;
		SetNumberDigits(nd);
		GetMemory(nd);
		for (int i = 1; i <= nd; ++i) {
			SetDigit(i, OldDigits[i-1]);
		}
		delete [] OldDigits;
	}
}

// Scale(): multiply the BigNum object by an int object
BigNum BigNum::Scale(int n) const {
	assert(n >= 0);
	if (n == 0)
		return 0;
	else if (n == 1)
		return *this;
	else {
		BigNum Product;
		int i = 0;
		while (n) {
			int Digit = n % 10;
			BigNum Term;
			Term.ResizeMemory(GetNumberDigits() + 1);
			Term.SetNumberDigits(GetNumberDigits() + 1);
			int Carry = 0;
			for (int j = 1; j <= GetNumberDigits(); ++j){
				int d = (Digit*GetDigit(j) + Carry) % 10;
				Carry = (Digit*GetDigit(j) + Carry) / 10;
				Term.SetDigit(j, d);
			}
			Term.SetDigit(Term.GetNumberDigits(),Carry);
			Term = Term.LeftDigitShift(i);
			Product = Product + Term;
			n = n / 10;
			++i;
		}
		Product.RemoveLeadingZeros();
		return Product;
	}
}


